import numpy as np
from modshogun import KSM_MAXIMIZE_POWER
from modshogun import SerializableAsciiFile
from modshogun import GaussianKernel
from modshogun import WrappedObjectArray
from modshogun import GaussianBlobsDataGenerator
from modshogun import LinearTimeMMD
from modshogun import CombinedKernel

features_p = GaussianBlobsDataGenerator()
features_q = GaussianBlobsDataGenerator()

#![create_instance]
mmd = LinearTimeMMD()
kernel = GaussianKernel(10, 1)
mmd.set_kernel(kernel)
mmd.set_p(features_p)
mmd.set_q(features_q)
mmd.set_num_samples_p(1000)
mmd.set_num_samples_q(1000)
alpha = 0.05
#![create_instance]

#![set_burst]
mmd.set_num_blocks_per_burst(1000)
#![set_burst]

#![estimate_mmd]
statistic = mmd.compute_statistic()
#![estimate_mmd]

#![perform_test]
threshold = mmd.compute_threshold(alpha)
p_value = mmd.compute_p_value(statistic)
#![perform_test]

#![add_kernels]
kernel1 = GaussianKernel(10, 0.1)
kernel2 = GaussianKernel(10, 1)
kernel3 = GaussianKernel(10, 10)
mmd.add_kernel(kernel1)
mmd.add_kernel(kernel2)
mmd.add_kernel(kernel3)
#![add_kernels]

#![enable_train_test_mode]
mmd.set_train_test_mode(True)
mmd.set_train_test_ratio(1)
#![enable_train_test_mode]

#![select_kernel_single]
mmd.set_kernel_selection_strategy(KSM_MAXIMIZE_POWER)
mmd.select_kernel()
learnt_kernel_single = GaussianKernel.obtain_from_generic(mmd.get_kernel())
width = learnt_kernel_single.get_width()
#![select_kernel_single]

#![select_kernel_combined]
mmd.set_kernel_selection_strategy(KSM_MAXIMIZE_POWER, True)
mmd.select_kernel()
learnt_kernel_combined = CombinedKernel.obtain_from_generic(mmd.get_kernel())
weights = learnt_kernel_combined.get_subkernel_weights()
#![select_kernel_combined]

#![perform_test_optimized]
statistic_optimized = mmd.compute_statistic()
p_value_optimized = mmd.compute_p_value(statistic)
#![perform_test_optimized]

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("linear_time_mmd.dat", 'w')
__sg_storage.append_wrapped_real(alpha, "alpha")
__sg_storage.append_wrapped_real(statistic, "statistic")
__sg_storage.append_wrapped_real(threshold, "threshold")
__sg_storage.append_wrapped_real(p_value, "p_value")
__sg_storage.append_wrapped_real(width, "width")
__sg_storage.append_wrapped_real_vector(weights, "weights")
__sg_storage.append_wrapped_real(statistic_optimized, "statistic_optimized")
__sg_storage.append_wrapped_real(p_value_optimized, "p_value_optimized")
__sg_storage.save_serializable(__sg_storage_file)
